# coding: utf8
from __future__ import unicode_literals, print_function

from collections import Counter
import sys

from .util import MESSAGES, COLORS, ICONS, wrap, supports_ansi, locale_escape
from .util import color as _color


class Printer(object):
    def __init__(self, pretty=True, no_print=False, colors=None, icons=None,
                 line_max=80, ignore_warnings=False):
        """Initialize the command-line printer.

        pretty (bool): Pretty-print output (colors, icons).
        no_print (bool): Don't actually print, just return.
        colors (dict): Add or overwrite color values, name mapped to value.
        icons (dict): Add or overwrite icons. Name mapped to unicode icon.
        line_max (int): Maximum line length (for divider).
        ignore_warnings (bool): Do not output messages of type MESSAGE.WARN.
        RETURNS (Printer): The initialized printer.
        """
        self._counts = Counter()
        self.pretty = pretty
        self.no_print = no_print
        self.supports_ansi = supports_ansi()
        self.ignore_warnings = ignore_warnings
        self.line_max = line_max
        self.colors = dict(COLORS)
        self.icons = dict(ICONS)
        if colors:
            self.colors.update(colors)
        if icons:
            self.icons.update(icons)

    @property
    def counts(self):
        """Get the counts of how often the special printers were fired,
        e.g. MESSAGES.GOOD. Can be used to print an overview like "X warnings".
        """
        return self._counts

    def good(self, text='', show=True):
        """Print a success message."""
        return self._get_msg(text, style=MESSAGES.GOOD, show=show)

    def fail(self, text='', show=True):
        """Print an error message."""
        return self._get_msg(text, style=MESSAGES.FAIL, show=show)

    def warn(self, text='', show=True):
        """Print a warning message."""
        return self._get_msg(text, style=MESSAGES.WARN, show=show)

    def info(self, text='', show=True):
        """Print an error message."""
        return self._get_msg(text, style=MESSAGES.INFO, show=show)

    def text(self, text='', color=None, icon=None, show=True):
        """Print a message.

        text (unicode): The text to print.
        color (unicode / int): Foreground color.
        icon (unicode): Name of icon to add.
        show (bool): Whether to print or not. Can be used to only output
            messages under certain condition, e.g. if --verbose flag is set.
        """
        if not show:
            return
        if self.pretty and self.supports_ansi:
            color = self.colors.get(color)
            icon = self.icons.get(icon)
            text = locale_escape('{} {}'.format(icon, text) if icon else text)
            text = _color(text, fg=color)
        if self.no_print:
            return text
        print(text)

    def divider(self, text='', char='=', show=True):
        """Print a divider with a headline:
        ============================= Headline here ============================

        text (unicode): Headline text. If empty, only the line is printed.
        char (unicode): Line character to repeat, e.g. =.
        show (bool): Whether to print or not.
        """
        if len(char) != 1:
            raise ValueError("Divider chars need to be one character long. "
                             "Received: {}".format(char))
        if self.pretty:
            deco = char * (round((self.line_max - len(text)) / 2) - 2)
            text = ' {} '.format(text) if text else ''
            text = _color('\n{deco}{text}{deco}'.format(deco=deco, text=text), bold=True)
        if len(text) < self.line_max:
            text = text + char * (self.line_max - len(text))
        if self.no_print:
            return text
        print(text)

    def _get_msg(self, text, style=None, show=None):
        if self.ignore_warnings and style == MESSAGES.WARN:
            show = False
        self._counts[style] += 1
        return self.text(text, color=style, icon=style, show=show)


def print_message(*texts, **kwargs):
    """Print formatted message.

    *texts (unicode): Texts to print. Each argument is rendered as paragraph.
    **kwargs: 'title' becomes headline. exits=1 performs sys exit.
    """
    exits = kwargs.get('exits')
    indent = kwargs.get('indent', 4)
    nowrap = kwargs.get('nowrap', False)
    title = kwargs.get('title', None)
    title_tpl = '{}\n'
    if nowrap:
        title = title_tpl.format(title) if title else ''
        message = '\n\n'.join(texts)
    else:
        title = title_tpl.format(wrap(title, indent=indent)) if title else ''
        message = '\n\n'.join([wrap(text, indent=indent) for text in texts])
    print('\n{}{}\n'.format(title, message))
    if exits is not None:
        sys.stdout.flush()
        sys.stderr.flush()
        sys.exit(exits)



#!/usr/bin/env python
# coding: utf8
from __future__ import unicode_literals

from pathlib import Path
from setuptools import setup, find_packages


def setup_package():
    package_name = 'wasabi'
    root = Path(__file__).parent.resolve()

    # Read in package meta from about.py
    about_path = root / package_name / 'about.py'
    with about_path.open('r', encoding='utf8') as f:
        about = {}
        exec(f.read(), about)

    # Get readme
    readme_path = root / 'README.md'
    with readme_path.open('r', encoding='utf8') as f:
        readme = f.read()

    setup(
        name=package_name,
        description=about['__summary__'],
        long_description=readme,
        author=about['__author__'],
        author_email=about['__email__'],
        url=about['__uri__'],
        version=about['__version__'],
        license=about['__license__'],
        packages=find_packages(),
        install_requires=[],
        zip_safe=False,
    )


if __name__ == '__main__':
    setup_package()



# coding: utf8
from __future__ import unicode_literals, print_function

import sys
import locale
import textwrap


class MESSAGES(object):
    GOOD = 'good'
    FAIL = 'fail'
    WARN = 'warn'
    INFO = 'info'


COLORS = {
    MESSAGES.GOOD: 2,
    MESSAGES.FAIL: 1,
    MESSAGES.WARN: 3,
    MESSAGES.INFO: 4,
    'red': 1,
    'green': 2,
    'yellow': 3,
    'blue': 4,
    'pink': 5,
    'cyan': 6,
    'white': 7,
    'grey': 8
}

ICONS = {
    MESSAGES.GOOD: '\u2714',
    MESSAGES.FAIL: '\u2718',
    MESSAGES.WARN: '\u26a0',
    MESSAGES.INFO: '\u2139'
}


def color(text, fg=None, bg=None, bold=False):
    """Color text by applying ANSI escape sequence.

    text (unicode): The text to be formatted.
    fg (unicode / int): Foreground color. String name or 0 - 256 (see COLORS).
    bg (unicode / int): Background color. String name or 0 - 256 (see COLORS).
    bold (bool): Format text in bold.
    RETURNS (unicode): The formatted text.
    """
    fg = COLORS.get(fg, fg)
    bg = COLORS.get(bg, bg)
    if not any([fg, bg, bold]):
        return text
    styles = []
    if bold:
        styles.append('1')
    if fg:
        styles.append('38;5;{}'.format(fg))
    if bg:
        styles.append('48;5;{}'.format(bg))
    return '\x1b[{}m{}\x1b[0m'.format(';'.join(styles), text)


def wrap(text, wrap_max=80, indent=4):
    """Wrap text at given width using textwrap module.

    text (unicode): The text to wrap.
    wrap_max (int): Maximum line width, including indentation. Defaults to 80.
    indent (int): Number of spaces used for indentation. Defaults to 4.
    RETURNS (unicode): The wrapped text with line breaks.
    """
    indent = indent * ' '
    wrap_width = wrap_max - len(indent)
    text = to_string(text)
    return textwrap.fill(text, width=wrap_width, initial_indent=indent,
                         subsequent_indent=indent, break_long_words=False,
                         break_on_hyphens=False)


def locale_escape(string, errors='replace'):
    """Mangle non-supported characters, for savages with ASCII terminals.

    string (unicode): The string to escape.
    errors (unicode): The str.encode errors setting. Defaults to `'replace'`.
    RETURNS (unicode): The escaped string.
    """
    encoding = locale.getpreferredencoding()
    string = to_string(string)
    string = string.encode(encoding, errors).decode('utf8')
    return string


def supports_ansi():
    """Returns True if the running system's terminal supports ANSI escape
    sequences for color, formatting etc. and False otherwise. Inspired by
    Django's solution – hacky, but an okay approximation.
    """
    # See: https://stackoverflow.com/q/7445658/6400719
    plat = sys.platform
    supported_platform = plat != 'Pocket PC' and (plat != 'win32' or
                                                  'ANSICON' in os.environ)
    if not supported_platform:
        return False
    return True


def to_string(text):
    """Minimal compat helper to make sure text is unicode. Mostly used to
    convert Paths and other Python objects.

    text: The text/object to be converted.
    RETURNS (unicode): The converted string.
    """
    is_python2 = sys.version_info[0] == 2
    if is_python2:
        basestring_ = basestring
    else:
        basestring_ = str
    if not isinstance(text, basestring_):
        if is_python2:
            text = str(text).decode('utf8')
        else:
            text = str(text)
    return text



__title__ = 'wasabi'
__version__ = '0.0.1'
__summary__ = 'A lightweight console printing and formatting toolkit'
__uri__ = 'https://ines.io'
__author__ = 'Ines Montani'
__email__ = 'ines@explosion.ai'
__license__ = 'MIT'
